from typing import Optional

from pydantic import BaseModel

from pycricinfo.source_models.api.common import Link, RefMixin


class Address(BaseModel):
    city: str
    state: Optional[str] = None
    zipCode: Optional[str] = None
    country: str
    summary: str


class Venue(BaseModel):
    id: str
    fullName: str
    shortName: str
    address: Address
    capacity: int
    grass: bool
    images: list[RefMixin]
    links: list[Link]



import requests
from fastapi import APIRouter, Depends, Path, Query, status

from pycricinfo.output_models.scorecard import CricinfoScorecard
from pycricinfo.search.call_cricinfo_api import get_match_basic, get_play_by_play, get_player, get_scorecard, get_team
from pycricinfo.source_models.api.commentary import CommentaryItem
from pycricinfo.source_models.api.match_basic import MatchBasic
from pycricinfo.source_models.api.player import Player
from pycricinfo.source_models.api.team import TeamFull

router = APIRouter(prefix="/wrapper", tags=["wrapper"])


class PageAndInningsQueryParameters:
    def __init__(
        self,
        page: int | None = Query(1, description="Which page of data to return"),
        innings: int | None = Query(1, description="Which innings of the game to get data from"),
    ):
        self.page = page
        self.innings = innings


@router.get(
    "/team/{team_id}", responses={status.HTTP_200_OK: {"description": "The Team data"}}, summary="Get Team data"
)
async def team(team_id: int = Path(description="The Team ID")) -> TeamFull:
    return get_team(team_id)


@router.get("/player/{player_id}", responses={status.HTTP_200_OK: {"description": "The Player"}}, summary="Get Player")
async def player(player_id: int = Path(description="The Player ID")) -> Player:
    return get_player(player_id)


@router.get(
    "/match/{match_id}",
    responses={status.HTTP_200_OK: {"description": "The basic match data"}},
    summary="Get basic match data from the '/events' API",
)
async def match_basic(match_id: int = Path(description="The Match ID")) -> MatchBasic:
    return get_match_basic(match_id)


@router.get(
    "/match/{match_id}/team/{team_id}",
    responses={status.HTTP_200_OK: {"description": "The basic match data"}},
    summary="Get a match's Team",
)
async def get_match_team(
    match_id: int = Path(description="The Match ID"), team_id: int = Path(description="The Team ID")
):
    response = requests.get(
        f"http://core.espnuk.org/v2/sports/cricket/leagues/0/events/{match_id}/competitions/{match_id}/competitors/{team_id}"
    ).json()
    return response


@router.get(
    "/match_summary/{match_id}",
    responses={status.HTTP_200_OK: {"description": "The match summary"}},
    summary="Get a match summary",
)
async def scorecard(match_id: int = Path(description="The Match ID")) -> CricinfoScorecard:
    return get_scorecard(match_id)


@router.get(
    "/match/{match_id}/play_by_play",
    responses={status.HTTP_200_OK: {"description": "The match summary"}},
    summary="Get a page of ball-by-ball data",
)
async def match_play_by_play(
    match_id: int = Path(description="The Match ID"), pi: PageAndInningsQueryParameters = Depends()
) -> list[CommentaryItem]:
    return get_play_by_play(match_id, pi.page, pi.innings)


# TODO: Add League endpoint: http://core.espnuk.org/v2/sports/cricket/leagues/22588



from typing import Literal

from fastapi import APIRouter, Path, Query, status

from pycricinfo.search.seasons import get_match_types_in_season
from pycricinfo.search.series import extract_match_ids_from_series
from pycricinfo.source_models.pages.series import MatchResult, MatchType

router = APIRouter(prefix="/matches", tags=["matches"])


@router.get(
    "/season/{season_name}",
    responses={
        status.HTTP_200_OK: {
            "description": "A list of match types, each containing a list of series in that match type for this season"
        }
    },
    summary="Get a list of match types, each containing a list of series in that match type for this season",
)
async def match_types_in_season(
    season_name: int = Path(description='The name of the season to get matches for, e.g. "2024" or "2020-21"'),
    match_type_name: Literal[
        "Tests",
        "One-Day Internationals",
        "Twenty20 Internationals",
        "First-class",
        "List A",
        "Twenty20",
        "Women's One-Day Internationals",
        "Women's Twenty20 Internationals",
        "Youth Tests",
        "Youth One-Day Internationals",
        "Women's Twenty20",
        "Tour",
        "Minor tour",
        "Other matches",
        "Other one-day/limited-overs matches",
        "Other Twenty20 matches",
    ] = Query(default=None, description="Filter the response to just matches of the named type"),
) -> list[MatchType]:
    match_types = get_match_types_in_season(season_name)

    if match_type_name:
        match_types = [m for m in match_types if m.name.lower() == match_type_name.lower()]

    return match_types


@router.get(
    "/series/{series_id}",
    responses={status.HTTP_200_OK: {"description": "A list of IDs of the matches in the supplied series"}},
    summary="Get a list of IDs of the matches in the supplied series",
)
async def match_ids_in_series(series_id: int = Path(description="The ID of a series")) -> list[MatchResult]:
    return extract_match_ids_from_series(series_id)



from typing import Literal, Optional

from pydantic import AliasChoices, BaseModel, Field, computed_field

from pycricinfo.source_models.api.common import CCBaseModel
from pycricinfo.source_models.api.linescores import TeamInningsDetails
from pycricinfo.source_models.api.match_note import MatchNote
from pycricinfo.source_models.api.official import Official
from pycricinfo.source_models.api.roster import TeamLineup
from pycricinfo.source_models.api.team import TeamWithColorAndLogos
from pycricinfo.source_models.api.venue import Venue


class MatchCompetitor(CCBaseModel):
    id: int
    winner: bool
    team: TeamWithColorAndLogos
    score: str
    linescores: list[TeamInningsDetails]
    home_or_away: Literal["home", "away"] = Field(validation_alias=AliasChoices("home_or_away", "homeAway"))


class MatchStatus(CCBaseModel):
    summary: str


class MatchCompetiton(CCBaseModel):
    status: MatchStatus
    competitors: list[MatchCompetitor]
    limited_overs: bool


class MatchHeader(CCBaseModel):
    id: int
    name: str
    description: str
    short_name: str
    title: str
    competitions: list[MatchCompetiton]
    # TODO: links
    # TODO: leagues

    @computed_field
    @property
    def summary(self) -> bool:
        return self.competitions[0].status.summary

    @computed_field
    @property
    def competition(self) -> MatchCompetiton:
        return self.competitions[0]

    def get_batting_linescore_for_period(self, period: int) -> tuple[TeamWithColorAndLogos, TeamInningsDetails]:
        for competitor in self.competition.competitors:
            for linescore in competitor.linescores:
                if linescore.period == period and linescore.is_batting:
                    return competitor.team, linescore


class MatchInfo(BaseModel):
    venue: Venue
    attendance: Optional[int] = None
    officials: list[Official]


class Match(CCBaseModel):
    notes: list[MatchNote]
    game_info: MatchInfo
    # TODO: add debuts
    rosters: list[TeamLineup]
    header: MatchHeader
